//---------------------------------------------------------------------------------------//
//-----------------------------------------Declarations----------------------------------//
//---------------------------------------------------------------------------------------//
#include <Adafruit_NeoPixel.h>
#include <ESP32Time.h>
#include <WiFi.h>

#define LED_PIN 16
#define LED_COUNT 100

Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

ESP32Time rtc(0);

bool isPressed = false;
bool wifiBug = false;

//------------------Wifi Setup-------------------
const char* ssid     = "ENTER SSID HERE";
const char* password = "ENTER PASSWORD HERE";

const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = -18000;
const int   daylightOffset_sec = 3600;

//---------------------------------------------------------------------------------------//
//--------------------------------------------Setup--------------------------------------//
//---------------------------------------------------------------------------------------//
void setup() {
  Serial.begin(115200);

  //------------------Wifi Setup-------------------
  wifi();

  //-------------------LED Setup--------------------
  strip.begin();
  strip.show();

  pinMode(16, OUTPUT);   // digital sensor is on digital pin 2
  pinMode(19, INPUT_PULLUP);
  pinMode(18, INPUT_PULLUP);
  pinMode(17, INPUT_PULLUP);
}

//---------------------------------------------------------------------------------------//
//------------------------------------------Variables------------------------------------//
//---------------------------------------------------------------------------------------//
//------------------Colors------------------
uint32_t white = strip.Color(255, 255, 255);
uint32_t min1 = strip.Color(255, 255, 255);
uint32_t min2 = strip.Color(255, 0, 0);
uint32_t min3 = strip.Color(0, 255, 0);
uint32_t min4 = strip.Color(0, 0, 255);
uint32_t min5 = strip.Color(255, 255, 0);

//------------------Words-------------------
int Its[] = {97, 3};
int SHU[] = {7, 3};
int Oclock[] = {0, 6};
int Past[] = {64, 4};
int To[] = {68, 2};
int A[] = {95, 1};
int AM[] = {92, 2};
int PM[] = {90, 2};

//------------------Minutes-----------------
int Five[] = {70, 4};
int Ten[] = {87, 3};
int Quarter[] = {80, 7};
int Twenty[] = {74, 6};
int Half[] = {60, 4};

//------------------Hours-------------------
int hours[12][3] = {
  {57, 3},  //One
  {54, 3},  //Two
  {40, 5},  //Three
  {36, 4},  //Four
  {20, 4},  //Five
  {24, 3},  //Six
  {45, 5},  //Seven
  {10, 5},  //Eight
  {50, 4},  //Nine
  {27, 3},  //Ten
  {30, 6},  //Eleven
  {14, 6}   //Twelve
};

//---------------------------------------------------------------------------------------//
//------------------------------------------Functions------------------------------------//
//---------------------------------------------------------------------------------------//
//--------------------Connect to wifi----------------------
void wifi(){
  int timeout = 0;
  wifiBug = false;
  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
      timeout ++;
      if (timeout == 10) {
        WiFi.disconnect(true);
        WiFi.mode(WIFI_OFF);
        wifiBug = true;
        return; }
  }
  Serial.println(" CONNECTED");
  
  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
}

void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}

//--------------------Light an hour----------------------
void hourLight(int hour) {
  strip.fill(white, hours[hour - 1][0], hours[hour - 1][1]);
}

//------------------Light a word white-------------------
void staticLight(int light[]) {
  strip.fill(white, light[0], light[1]);
}

//------------------Light the minutes--------------------
void minuteLight(int light[], int minute) {
  minute %= 10;
  switch (minute) {
    case 0:
    case 5:
      strip.fill(min1, light[0], light[1]);
      break;
    case 1:
    case 6:
      strip.fill(min2, light[0], light[1]);
      break;
    case 2:
    case 7:
      strip.fill(min3, light[0], light[1]);
      break;
    case 3:
    case 8:
      strip.fill(min4, light[0], light[1]);
      break;
    case 4:
    case 9:
      strip.fill(min5, light[0], light[1]);
      break;
  }
}

//---------------------------------------------------------------------------------------//
//--------------------------------------------Main---------------------------------------//
//---------------------------------------------------------------------------------------//
void loop() {
  strip.clear();

  int hour = rtc.getHour(true);
  int minute = rtc.getMinute();
  int second = rtc.getSecond();

  int reset = digitalRead(19);
  int hourVal = digitalRead(18);
  int minuteVal = digitalRead(17);

  //----------------------------------------Reset Button---------------------------------//
  if (reset == LOW)
    wifi();

  //----------------------------------------Hour Button----------------------------------//
  if ((hourVal == LOW) && (isPressed == false)) {
    isPressed = true;
    if (wifiBug == false)
      rtc.setTime(second, minute - 4, hour - 1, 1, 1, 23); //Adjusts the time
    else if (wifiBug == true)
      rtc.setTime(second, minute - 4, hour, 1, 1, 23); //Adjusts the time
  }

  //---------------------------------------Minute Button---------------------------------//
  if ((minuteVal == LOW) && (isPressed == false)) {
    isPressed = true;
    if (wifiBug == false)
      rtc.setTime(0, minute - 3, hour - 2, 1, 1, 23); //Adjusts the time
    else if (wifiBug == true)
      rtc.setTime(second, minute - 3, hour - 1, 1, 1, 23); //Adjusts the time
  }

  //---------------------------------------Check Buttons---------------------------------//
  if ((minuteVal == HIGH) && (hourVal == HIGH))
    isPressed = false;

  //---------------------------------------Static Lights---------------------------------//
  staticLight(Its);

  String ampm = rtc.getAmPm();
  if ((hour == 11) && (minute >= 35)) {
    staticLight(PM);
    Serial.print("PM ");}
  else if ((hour == 23) && (minute >= 35)) {
    staticLight(AM);
    Serial.print("AM ");}
  else if (ampm == "AM") {
    staticLight(AM);
    Serial.print("AM ");}
  else if (ampm == "PM"){
    staticLight(PM);
    Serial.print("PM ");}

  //-------------------------------------Lights the Second-------------------------------//
  if (second % 2 == 0)
    staticLight(SHU);

  //--------------------------------------Lights the Hour--------------------------------//
  if (hour > 12)
    hour -= 12;
  if (hour == 0)
    hour = 12;
  if ((hour == 12) && (minute >= 35))
    hourLight(1);
  else if (minute >= 35)
    hourLight(hour + 1);
  else
    hourLight(hour);

  //-------------------------------------Lights the Minute-------------------------------//
  switch (minute) {
    case 0 ... 4:
      minuteLight(Oclock, minute);
      break;
    case 5 ... 9:
      minuteLight(Five, minute);
      minuteLight(Past, minute);
      break;
    case 10 ... 14:
      minuteLight(Ten, minute);
      minuteLight(Past, minute);
      break;
    case 15 ... 19:
      minuteLight(A, minute);
      minuteLight(Quarter, minute);
      minuteLight(Past, minute);
      break;
    case 20 ... 24:
      minuteLight(Twenty, minute);
      minuteLight(Past, minute);
      break;
    case 25 ... 29:
      minuteLight(Twenty, minute);
      minuteLight(Five, minute);
      minuteLight(Past, minute);
      break;
    case 30 ... 34:
      minuteLight(Half, minute);
      minuteLight(Past, minute);
      break;
    case 35 ... 39:
      minuteLight(Twenty, minute);
      minuteLight(Five, minute);
      minuteLight(To, minute);
      break;
    case 40 ... 44:
      minuteLight(Twenty, minute);
      minuteLight(To, minute);
      break;
    case 45 ... 49:
      minuteLight(A, minute);
      minuteLight(Quarter, minute);
      minuteLight(To, minute);
      break;
    case 50 ... 54:
      minuteLight(Ten, minute);
      minuteLight(To, minute);
      break;
    case 55 ... 59:
      minuteLight(Five, minute);
      minuteLight(To, minute);
      break;
  }

  //-----------------------------------------Debugging-----------------------------------//
  Serial.print(rtc.getHour(true));
  Serial.print(':');
  Serial.print(rtc.getHour());
  Serial.print(':');
  Serial.print(hour);
  Serial.print(':');
  Serial.print(minute);
  Serial.print(':');
  Serial.print(second);
  Serial.print(' ');
  Serial.print(rtc.getAmPm());
  Serial.println();

  //----------------------------------------Show Strip-----------------------------------//
  strip.show();
}
